package error;

import java.text.MessageFormat;
import java.util.function.Consumer;

/**
 * Prints error messages and potentially terminates the application.
 */
public class ErrorManager {

    public static final int CRITICAL_LEVEL = 3;

    public static final String ANSI_RESET = "\u001B[0m";
    public static final String ANSI_RED = "\u001B[31m";
    public static final String ANSI_YELLOW = "\u001B[33m";
    public static final String ANSI_PURPLE = "\u001B[35m";

    private Runnable systemTerminatingFunction;
    private Consumer<String> outputFunction;

    public ErrorManager(Runnable systemTerminatingFunction, Consumer<String> outputFunction) {
        this.systemTerminatingFunction = systemTerminatingFunction;
        this.outputFunction = outputFunction;
    }

    public void generateError(LispException lispException) {
        String color = (lispException.getSeverity() >= CRITICAL_LEVEL) ? ANSI_PURPLE : ANSI_RED;
        String formattedMessage = MessageFormat.format("{0}error: {1}{2}\n", color, lispException.getMessage(),
                                                       ANSI_RESET);

        outputFunction.accept(formattedMessage);

        if (lispException.getSeverity() >= CRITICAL_LEVEL) {
            systemTerminatingFunction.run();
        }
    }

}
